@@ -327,6 +327,10 @@ class CommandMulti(CommandParallel):
 
         if parsed.fail_if_no_targets and len(self.targets) == 0:
             self.error(f'Multi: --fail-if-no-targets set, and {self.targets=}')
+        if not all_multi_tools:
+            possible_tools = self.all_handler_commands.get(command, [])
+            self.error(f'Multi: no tools to run for {command=}, available tools: {possible_tools}')
+
         util.info("Multi: About to run: ", end="")
 
         def get_pretty_targets_tuple_as_list(l:list):
