@@ -63,6 +63,8 @@ class CommandSim(CommandDesign):
             'sim-args': [],
             'sim-plusargs': [], # lists are handled by 'set_arg(k,v)' so they append.
             'sim-library': [],
+            'compile-waivers': [],
+            'sim-waivers': [],
             'coverage': False,
             'waves': False,
             'waves-start': 0,
@@ -142,6 +144,8 @@ class CommandSim(CommandDesign):
             self.create_work_dir()
             self.run_dep_commands()
             self.do_it()
+            self.run_post_tool_dep_commands()
+            self.report_pass_fail()
         return unparsed
 
 
@@ -195,6 +199,9 @@ class CommandSim(CommandDesign):
 
             clist = list(obj).copy()
             tee_fpath = getattr(obj, 'tee_fpath', None)
+            work_dir = getattr(obj, 'work_dir', None)
+            if not work_dir:
+                work_dir = self.args['work-dir']
 
             util.debug(f'run_commands_check_logs: {clist=}, {tee_fpath=}')
 
@@ -204,24 +211,28 @@ class CommandSim(CommandDesign):
             if log_filename:
                 log_fname = log_filename
 
+
             _, stdout, _ = self.exec(
-                work_dir=self.args['work-dir'], command_list=clist, tee_fpath=tee_fpath
+                work_dir=work_dir, command_list=clist, tee_fpath=tee_fpath
             )
 
             if check_logs and log_fname:
                 # Note this call will check on stdout if not GUI, not opening the log_fname,
                 # but if this is GUI we normally lose stdout and have to open the log.
-                gui_mode = self.args.get('gui', False)
-                file_contents_str = '' if gui_mode else stdout
+                if self.args.get('gui', False):
+                    file_contents_str = ''
+                else:
+                    file_contents_str = stdout
+
                 self.check_logs_for_errors(
-                    filename=os.path.join(self.args['work-dir'], log_fname),
+                    filename=os.path.join(work_dir, log_fname),
                     file_contents_str=file_contents_str,
                     bad_strings=bad_strings, must_strings=must_strings,
                     use_bad_strings=use_bad_strings, use_must_strings=use_must_strings
                 )
             if log_fname:
                 self.artifacts_add(
-                    name=os.path.join(self.args['work-dir'], log_fname),
+                    name=os.path.join(work_dir, log_fname),
                     typ='text', description='Simulator stdout/stderr log file'
                 )
 
@@ -250,6 +261,7 @@ class CommandSim(CommandDesign):
         tool = self.args.get('tool', None)
         # Certain args are allow-listed here
         deps_file_args = []
+        print(f'SUPER DREW DEBUG: {self.get_command_line_args()=}')
         for a in self.get_command_line_args():
             if any(a.startswith(x) for x in [
                     '--compile-args',
@@ -262,7 +274,8 @@ class CommandSim(CommandDesign):
                     '--stop-',
                     '--lint-',
                     '--verilate',
-                    '--verilator']):
+                    '--verilator',
+                    '--cocotb-test-']):
                 deps_file_args.append(a)
 
         export_obj.run(
@@ -400,7 +413,7 @@ class CommandSim(CommandDesign):
         '''
 
         all_lists = [] # list - of - (command-list)
-        if self.has_dep_shell_commands:
+        if self.has_pre_compile_dep_shell_commands:
             all_lists = [
                 ['./pre_compile_dep_shell_commands.sh']
             ]
@@ -423,6 +436,11 @@ class CommandSim(CommandDesign):
                                           line_breaks=simulate_line_breaks)
             all_lists.append(['./' + simulate_sh_fname])
 
+        if self.has_post_tool_dep_shell_commands:
+            all_lists = [
+                ['./post_tool_dep_shell_commands.sh']
+            ]
+
         util.write_shell_command_file(dirpath=self.args['work-dir'], filename='all.sh',
                                       command_lists=all_lists)
 
@@ -459,3 +477,41 @@ class CommandSim(CommandDesign):
     def get_post_simulate_command_lists(self, **kwargs) -> list:
         ''' Returns a list of lists (list of command lists).'''
         raise NotImplementedError
+
+    def add_waves_pkg_file(self) -> None:
+        '''If --waves is present, and one of --waves-fst or --waves-vcd or --dump-vcd, and
+
+        the user is missing any $dumpfile(), then adds a pre-written
+        SystemVerilog package to their source code. Note that individual tools have
+        to call this prior to their compile step, CommandSim does not run this method for
+        you.
+        '''
+        if not self.args['waves']:
+            return
+        if not any(self.args.get(x, False) for x in ('waves-fst', 'waves-vcd', 'dump-vcd')):
+            return
+        found_dumpfile = False
+        for fname in self.files_v + self.files_sv:
+            if found_dumpfile:
+                break
+            with open(fname, encoding='utf-8') as f:
+                for line in f.readlines():
+                    if '$dumpfile' in line:
+                        found_dumpfile = True
+                        break
+
+        if not found_dumpfile:
+            thispath = os.path.dirname(__file__)
+            file_to_add = os.path.join(thispath, '..', '_waves_pkg.sv')
+            util.info(f'--waves arg present, no $dumpfile found, adding SV file: {file_to_add}')
+            self.add_file(file_to_add)
+
+        # register .vcd or .fst artifacts:
+        util.artifacts.add_extension(
+            search_paths=self.args['work-dir'], file_extension='fst',
+            typ='waveform', description='Simulation Waveform FST (Fast Signal Trace) file'
+        )
+        util.artifacts.add_extension(
+            search_paths=self.args['work-dir'], file_extension='vcd',
+            typ='waveform', description='Simulation Waveform VCD (Value Change Dump) file'
+        )
