@@ -33,6 +33,7 @@ SUPPORTED_TARGET_TABLE_KEYS = set([
     'defines',
     'parameters',
     'incdirs',
+    'plusargs',
     'top',
     'deps',
     'reqs',
@@ -59,14 +60,22 @@ SUPPORTED_TAG_KEYS = set([
     'additive-config-tools',
 ])
 
+COMMAND_ATTRIBUTES = {
+    # with default values
+    'run-from-work-dir': True,
+    'filepath-subst-target-dir': True,
+
+    'run-after-tool': False,   # default is to run before any tool compile step.
+    'dirpath-subst-target-dir': False,
+    'var-subst-args': False,
+    'var-subst-os-env': False,
+    'tee': None,
+}
+
 SUPPORTED_COMMAND_KEYS = set([
     'shell',
-    'work-dir-add-srcs', 'work-dir-add-sources',
     'peakrdl',
-    'run-from-work-dir',     # default True
-    'filepath-subst-target-dir', # default True
-    'dirpath-subst-target-dir',  # default False
-    'var-subst-args',
-    'var-subst-os-env',
-    'tee',
+    'work-dir-add-srcs', 'work-dir-add-sources'
 ])
+for key in COMMAND_ATTRIBUTES:
+    SUPPORTED_COMMAND_KEYS.add(key)
