@@ -4,11 +4,13 @@ a DEPS markup files targets (applying deps, reqs, commands, tags, incdirs, defin
 CommandDesign ref object
 '''
 
+import argparse
 import os
 
 from opencos import files
 from opencos import eda_config
-from opencos.util import debug, info, warning, error
+from opencos.util import debug, info, warning, error, read_tokens_from_dot_f, \
+    patch_args_for_dir, load_env_file
 from opencos.utils.str_helpers import dep_str2list
 from opencos.deps.deps_file import deps_target_get_deps_list
 from opencos.deps.deps_commands import deps_commands_handler
@@ -48,6 +50,7 @@ class DepsProcessor: # pylint: disable=too-many-instance-attributes
         self.target_path = target_path
         self.target_node = target_node # for debug
         self.deps_file = deps_file # for debug
+        self.deps_dir, _ = os.path.split(deps_file)
         self.caller_info = caller_info
 
         assert isinstance(deps_entry, dict), \
@@ -74,7 +77,7 @@ class DepsProcessor: # pylint: disable=too-many-instance-attributes
         ).get('command_handler', {}).keys()
 
 
-    def apply_defines(self, defines_dict: dict):
+    def apply_defines(self, defines_dict: dict) -> None:
         '''Given defines_dict, applies them to our self.command_design_ref obj'''
         if not isinstance(defines_dict, dict):
             self.error(f"{defines_dict=} is not type dict, can't apply defines,",
@@ -92,7 +95,19 @@ class DepsProcessor: # pylint: disable=too-many-instance-attributes
                 self.command_design_ref.process_plusarg(f'+define+{k}={v}')
 
 
-    def apply_parameters(self, parameters_dict: dict):
+    def apply_plusargs(self, plusargs_dict: dict) -> None:
+        '''Given plusarsg_dict, applies them to our self.command_design_ref obj'''
+        if not isinstance(plusargs_dict, dict):
+            self.error(f"{plusargs_dict=} is not type dict, can't apply plusargs,",
+                       f"in {self.caller_info}")
+        for k,v in plusargs_dict.items():
+            if v is None or v == '':
+                self.command_design_ref.process_plusarg(f'+{k}')
+            else:
+                self.command_design_ref.process_plusarg(f'+{k}={v}')
+
+
+    def apply_parameters(self, parameters_dict: dict) -> None:
         '''Given parameters_dict, applies them to our self.command_design_ref obj'''
         if not isinstance(parameters_dict, dict):
             self.error(f"{parameters_dict=} is not type dict, can't apply defines,",
@@ -107,7 +122,7 @@ class DepsProcessor: # pylint: disable=too-many-instance-attributes
                 )
 
 
-    def apply_incdirs(self, incdirs_list:list):
+    def apply_incdirs(self, incdirs_list:list) -> None:
         '''Given incdirs_list, applies them to our self.command_design_ref obj'''
         if not isinstance(incdirs_list, (str, list)):
             self.error(f"{incdirs_list=} is not type str/list, can't apply incdirs",
@@ -150,7 +165,9 @@ class DepsProcessor: # pylint: disable=too-many-instance-attributes
         return tokens
 
 
-    def apply_args(self, args_list:list) -> list:
+    def apply_args( # pylint: disable=too-many-locals,too-many-branches
+            self, args_list:list
+    ) -> list:
         '''Given args_list, applies them to our self.command_design_ref obj
 
         This will return unparsed args that weren't in the self.command_design_ref.args keys
@@ -162,10 +179,65 @@ class DepsProcessor: # pylint: disable=too-many-instance-attributes
                        f"in {self.caller_info}")
         tokens = dep_str2list(args_list)
 
+        # patch args relative to the DEPS (if self.deps_dir exists) so things like
+        # --build-tcl=<file> for relative <file> works when calling targets from any directory.
+        tokens = patch_args_for_dir(
+            tokens=tokens, patch_dir=self.deps_dir, caller_info=self.caller_info
+        )
+
         # We're going to run an ArgumentParser here, which is not the most efficient
         # thing to do b/c it runs on all of self.command_design_ref.args (dict) even
         # if we're applying a single token.
 
+        # Since some args (util.py, eda_config.py, eda.py) can only be handled from command
+        # line, it would be nice if -f or --input-file is handled from DEPS, so we'll special
+        # case that now. Recursively resolve -f / --input-file.
+        # Do similary for --env-file (also only supported in util.py)
+        parser = argparse.ArgumentParser(
+            prog='deps_processor -f/--input-file', add_help=False, allow_abbrev=False
+        )
+        parser.add_argument('--env-file', default=[], action='append',
+                            help=(
+                                "dotenv file(s) to pass ENV vars, (default: .env loaded first,"
+                                " subsequent files' vars override .env"
+                            ))
+        parser.add_argument('-f', '--input-file', default=[], action='append',
+                            help=(
+                                'Input .f file to be expanded as eda args, defines, incdirs,'
+                                ' files, or targets.'
+                            ))
+        try:
+            parsed, unparsed = parser.parse_known_args(tokens + [''])
+            tokens2 = list(filter(None, unparsed))
+        except argparse.ArgumentError:
+            error('deps_processor -f/--input-file, problem attempting to parse_known_args for:',
+                  f'{tokens}')
+            tokens2 = tokens
+
+        if parsed.input_file:
+            dotf_tokens = []
+            for filepath in parsed.input_file:
+                # Since this isn't command line, we have to assume the path is relative
+                # to this DEPS file.
+                if not os.path.isabs(filepath):
+                    filepath = os.path.join(self.deps_dir, filepath)
+                dotf_tokens.extend(read_tokens_from_dot_f(
+                    filepath=filepath, caller_info=self.caller_info, verbose=True
+                ))
+
+            # put the .f files before the unparsed args.
+            tokens2 = dotf_tokens + tokens2
+
+            # recurse until we've resolved nested .f files.
+            return self.apply_args(args_list=tokens2)
+
+        if parsed.env_file:
+            for env_file in parsed.env_file:
+                load_env_file(env_file)
+
+        # if no --input-file/--env-file values, keep parsing the remaining tokens2:
+        tokens = tokens2
+
         # We have to special-case anything with --tool[=value] in tokens, otherwise
         # the user may think they were allowed to set --tool, but in our flow the Command handler
         # (self.command_design_ref) has already been chosen, so setting the tool can have
@@ -180,11 +252,38 @@ class DepsProcessor: # pylint: disable=too-many-instance-attributes
         _, unparsed = self.command_design_ref.run_argparser_on_list(
             tokens=tokens
         )
+
         # Annoying, but check for plusargs in unparsed, and have referenced CommandDesign
         # or CommandSim class handle it with process_plusarg.
-        for arg in unparsed:
+        for arg in list(unparsed):
             if arg.startswith('+'):
                 self.command_design_ref.process_plusarg(plusarg=arg, pwd=self.target_path)
+                unparsed.remove(arg)
+
+        # For any leftover files, or targets, attempt to process those too:
+        for arg in list(unparsed):
+            # Since this isn't command line, we have to assume for files, the path is relative
+            # to this DEPS file.
+            if os.path.isabs(arg):
+                target = arg
+            else:
+                target = os.path.join(self.deps_dir, arg)
+
+            file_exists, fpath, forced_extension = files.get_source_file(target)
+            if file_exists:
+                _, file_ext = os.path.splitext(fpath)
+                if forced_extension or file_ext:
+                    self.command_design_ref.add_file(fpath, caller_info=self.caller_info,
+                                                     forced_extension=forced_extension)
+                    unparsed.remove(arg)
+
+            else:
+                if not os.path.isdir(target) and \
+                   self.command_design_ref.resolve_target_core(
+                       target=target, no_recursion=False, caller_info=self.caller_info,
+                       error_on_not_found=False
+                   ):
+                    unparsed.remove(arg)
 
         if unparsed:
             # This is only a warning - because things like CommandFlist may not have every
@@ -214,7 +313,9 @@ class DepsProcessor: # pylint: disable=too-many-instance-attributes
                 caller_info=self.caller_info
             )
 
-    def process_deps_entry(self):
+    def process_deps_entry( # pylint: disable=too-many-branches
+            self
+    ) -> list:
         '''Main entry point (after creating DepsProcessor obj) to resolve a deps target
 
         Example usage:
@@ -227,22 +328,6 @@ class DepsProcessor: # pylint: disable=too-many-instance-attributes
         This method will apply all target features to the CommandDesign ref object as
         we traverse.
 
-        Supported target keys:
-         -- tags (or equivalent, to support multiple define/incdir/deps for a target)
-            -- supports tag-name, with-tools, with-args, args, defines, incdirs, deps
-                 ** to be applied if a tool matches.
-            -- TODO(drew): other features in docs/DEPS.md not yet implemented.
-         -- multi: ignore-this-target: - commands (handled in eda.py CommandMulti.resolve_target)
-         -- Named eda commands
-            -- (partially done) sim or other eda commands (eda.py command specific things)
-                basically, check the command, and apply/merge values to 'entry'?
-         -- args
-         -- defines
-         -- incdirs
-         -- top.
-         -- commands (not in deps)
-         -- deps
-
         TODO(drew): This does not yet support conditional inclusions based on defines,
          like the old DEPS files did with pattern:
             SOME_DEFINE ?  dep_if_define_present : dep_if_define_not_present
@@ -268,6 +353,8 @@ class DepsProcessor: # pylint: disable=too-many-instance-attributes
                 remaining_deps_list += self.process_tags()
             elif key == 'defines':
                 self.process_defines()
+            elif key == 'plusargs':
+                self.process_plusargs()
             elif key == 'parameters':
                 self.process_parameters()
             elif key == 'incdirs':
@@ -444,6 +531,10 @@ class DepsProcessor: # pylint: disable=too-many-instance-attributes
                         # apply defines:
                         self.apply_defines(value.get('defines', {}))
 
+                    if key == 'plusargs':
+                        # apply plusargs:
+                        self.apply_plusargs(value.get('plusargs', {}))
+
                     elif key == 'parameters':
                         self.apply_parameters(value.get('parameters', {}))
 
@@ -522,7 +613,7 @@ class DepsProcessor: # pylint: disable=too-many-instance-attributes
         return ret_deps_added_from_tags
 
 
-    def process_defines(self):
+    def process_defines(self) -> None:
         '''Returns None, applies defines (dict, if any) from self.deps_entry to
         self.command_design_ref.'''
 
@@ -538,7 +629,28 @@ class DepsProcessor: # pylint: disable=too-many-instance-attributes
 
         self.apply_defines(entry_defines)
 
-    def process_parameters(self):
+
+    def process_plusargs(self) -> None:
+        '''Returns None, applies plusargs (dict, if any) from self.deps_entry to
+        self.command_design_ref.
+
+        These work w/ the same rules as defines (no value, or value int/str)
+        '''
+
+        # Plusargs:
+        # apply command specific plusargs, with higher priority than the a
+        # deps_entry['sim']['plusargs'] entry,
+        # do this with dict1.update(dict2):
+        entry_plusargs = {}
+        entry_plusargs.update(self.deps_entry.get('plusargs', {}))
+        entry_plusargs.update(self.entry_eda_command.get('plusargs', {}))
+        assert isinstance(entry_plusargs, dict), \
+            f'{entry_plusargs=} for in {self.caller_info} must be a dict'
+
+        self.apply_plusargs(entry_plusargs)
+
+
+    def process_parameters(self) -> None:
         '''Returns None, applies parameters (dict, if any) from self.deps_entry to
         self.command_design_ref.'''
 
