@@ -17,6 +17,9 @@ my_target_name:
   incdirs:                           <---- incdirs, optional array (or string)
     - ./
 
+  plusargs:                          <---- plusargs, optional table
+    some_plusarg: 32
+
   top: tb                            <---- top, optional string
 
   deps:
@@ -63,6 +66,7 @@ my_target_name:
   defines:                           <---- defines, optional table
   parameters:                        <---- parameters, optional table
   incdirs:                           <---- incdirs, optional array (or string)
+  plusargs:                          <---- plusargs, optional table
   top: tb                            <---- top, optional string
   deps:                              <---- TARGET_DEPS_CONTENTS schema
     - some_file.sv                   <---- string file name
@@ -85,6 +89,8 @@ my_target_name:
                                            directory path substitution relative to target dir
                                            (if substituted directory exists).
         tee:                         <---- string, filename to write logs to
+        run-after-tool:              <---- bool, default false. Set to true to run after any
+                                           EDA tools, or any command handlers have completed.
       - work-dir-add-sources:        <---- work-dir-add-sources, optional list (or string)
         - some_file_gen_from_sh.sv   <---- string filename that we created with sh command
       - peakrdl:                     <---- string peakrdl command for CSR generation
@@ -171,6 +177,7 @@ DEPS_COMMANDS_LIST = [
         Optional('var-subst-args'): bool,
         Optional('var-subst-os-env'): bool,
         Optional('run-from-work-dir'): bool,
+        Optional('run-after-tool'): bool,
         Optional('filepath-subst-target-dir'): bool,
         Optional('dirpath-subst-target-dir'): bool,
         Optional('tee'): Or(str, type(None)),
@@ -213,6 +220,9 @@ TARGET_EDA_COMMAND_ENTRY_TABLE = {
     Optional('defines'): {
         Optional(str): Or(str, int, type(None)),
     },
+    Optional('plusargs'): {
+        Optional(str): Or(str, int, type(None)),
+    },
     Optional('parameters'): {
         Optional(str): Or(str, int),
     },
@@ -238,6 +248,9 @@ TARGET_TAGS_TABLE = {
         Optional('defines'): {
             Optional(str): Or(str, int, type(None)),
         },
+        Optional('plusargs'): {
+            Optional(str): Or(str, int, type(None)),
+        },
         Optional('parameters'): {
             Optional(str): Or(str, int),
         },
@@ -259,6 +272,10 @@ TARGET_CONTENTS = Or(
         Optional('defines'): {
             Optional(str): Or(str, int, type(None)),
         },
+        # plusargs: table of key-value; value null or string
+        Optional('plusargs'): {
+            Optional(str): Or(str, int, type(None)),
+        },
         # parameters: table of key-value
         Optional('parameters'): {
             Optional(str): Or(str, int),
@@ -321,6 +338,9 @@ FILE_SIMPLIFIED = Schema(
                     Optional('defines'): {
                         Optional(str): Or(type(None), str),
                     },
+                    Optional('plusargs'): {
+                        Optional(str): Or(type(None), str),
+                    },
                     Optional('parameters'): {
                         Optional(str): str,
                     },
