@@ -20,10 +20,11 @@ from pathlib import Path
 
 import opencos
 from opencos import util, eda_config, eda_base
+from opencos.util import safe_emoji
 from opencos.eda_base import Tool, which_tool, get_eda_exec
 from opencos.utils import vsim_helper, vscode_helper
 from opencos.utils.subprocess_helpers import subprocess_run_background
-from opencos.utils import status_constants, str_helpers
+from opencos.utils import status_constants, str_helpers, subprocess_helpers
 
 # Configure util:
 util.progname = "EDA"
@@ -50,7 +51,7 @@ def init_config(
     # For key DEFAULT_HANDLERS, we'll update config['command_handler'] with
     # the actual class using importlib (via opencos.util)
 
-    eda_config.tool_try_add_to_path(tool)
+    eda_config.update_config_auto_tool_order_for_tool(tool=tool, config=config)
 
     config['command_handler'] = {}
     for _cmd, str_class in config['DEFAULT_HANDLERS'].items():
@@ -95,15 +96,15 @@ def usage(tokens: list, config: dict, command: str = "", tool: str = "") -> int:
 
     if command == "":
         print(
-"""
-Usage:
+f"""
+{safe_emoji("🔦 ")}Usage:
     eda [<options>] <command> [options] <files|targets, ...>
 """
         )
         print(get_all_commands_help_str(config=config))
         print(
-"""
-And <files|targets, ...> is one or more source file or DEPS markup file target,
+f"""
+{safe_emoji("❕ ")}where <files|targets, ...> is one or more source file or DEPS markup file target,
     such as .v, .sv, .vhd[l], .cpp files, or a target key in a DEPS.[yml|yaml|toml|json].
     Note that you can prefix source files with `sv@`, `v@`, `vhdl@` or `cpp@` to
     force use that file as systemverilog, verilog, vhdl, or C++, respectively.
@@ -163,10 +164,7 @@ def auto_tool_setup( # pylint: disable=too-many-locals,too-many-branches,too-man
       If so, updates config['auto_tools_order'][tool]['exe']
     '''
 
-
-    tool = eda_config.update_config_auto_tool_order_for_tool(
-        tool=tool, config=config
-    )
+    tool = eda_config.tool_arg_remove_path_information(tool)
 
     assert 'auto_tools_order' in config
     assert isinstance(config['auto_tools_order'], list)
@@ -275,9 +273,7 @@ def tool_setup(tool: str, config: dict, quiet: bool = False, auto_setup: bool =
 
     '''
 
-    tool = eda_config.update_config_auto_tool_order_for_tool(
-        tool=tool, config=config
-    )
+    tool = eda_config.tool_arg_remove_path_information(tool)
 
     if not quiet and not auto_setup:
         util.info(f"Setup for tool: '{tool}'")
@@ -389,6 +385,8 @@ def process_tokens( # pylint: disable=too-many-branches,too-many-statements,too-
     if not is_interactive:
         # Run init_config() now, we deferred it in main(), but only run it
         # for this tool (or tool=None to figure it out)
+        # This will handle any --tool=<name>=/path/to/bin also, so don't have to
+        # run tool_setup(..) on its own.
         config = init_config(
             config, tool=parsed.tool,
             run_auto_tool_setup=run_auto_tool_setup
@@ -406,13 +404,8 @@ def process_tokens( # pylint: disable=too-many-branches,too-many-statements,too-
             for arg in unparsed:
                 if not arg.startswith('-'):
                     command = arg
-        if parsed.tool:
-            tool_setup(parsed.tool, config=config)
         return usage(tokens=unparsed, config=config, command=command, tool=parsed.tool)
 
-    if parsed.tool:
-        tool_setup(parsed.tool, config=config)
-
     deferred_tokens = unparsed
     if not command:
         util.error("Didn't get a command!")
@@ -503,7 +496,8 @@ def check_command_handler_cls(command_obj:object, command:str, parsed_args) -> i
 def signal_handler(sig, frame) -> None: # pylint: disable=unused-argument
     '''Handles Ctrl-C, called by main_cli() if running from command line'''
     util.fancy_stop()
-    util.info('Received Ctrl+C...', start='\nINFO: [EDA] ')
+    util.error(f'{safe_emoji("❌ ")}Received Ctrl+C...', start='\nINFO: [EDA] ')
+    subprocess_helpers.cleanup_all()
     util.exit(-1)
 
 # **************************************************************
@@ -520,7 +514,8 @@ def main(*args):
 
     original_args = args.copy() # save before any parsing.
 
-    # Set global --debug, --quiet, --color  early before parsing other args:
+    # Set global --debug, --quiet, --color, -f/--iput-file, --envfile
+    # early before parsing other args:
     util_parsed, unparsed = util.process_tokens(args)
 
     util.debug(f"main: file: {os.path.realpath(__file__)}")
