@@ -15,10 +15,14 @@ import shutil
 import mergedeep
 
 from opencos import util
+from opencos.util import safe_emoji
 from opencos.utils.markup_helpers import yaml_safe_load, yaml_safe_writer
 
 class Defaults:
-    '''Defaults is a global placeholder for constants and supported features.'''
+    '''Defaults is a global namespace for constants and supported features.
+
+    Defaults.config_yml is set depending on search order for default eda_config[_defaults].yml
+    '''
 
     environ_override_config_yml = os.environ.get('EDA_CONFIG_YML', '')
     home_override_config_yml = os.path.join(
@@ -63,14 +67,15 @@ class Defaults:
         'simulate-args',
         'simulate-waves-args',
         'simulate-waivers',
+        'simulate-coverage-tcl',
         'coverage-args',
     ])
 
 EDA_OUTPUT_CONFIG_FNAME = 'eda_output_config.yml'
 
-if os.path.exists(Defaults.environ_override_config_yml):
+if os.path.isfile(Defaults.environ_override_config_yml):
     Defaults.config_yml = Defaults.environ_override_config_yml
-elif os.path.exists(Defaults.home_override_config_yml):
+elif os.path.isfile(Defaults.home_override_config_yml):
     Defaults.config_yml = Defaults.home_override_config_yml
 else:
     Defaults.config_yml = Defaults.opencos_config_yml
@@ -149,29 +154,9 @@ def update_config_auto_tool_order_for_tool(tool: str, config: dict) -> str:
     Input arg tool can be in the form (for example):
       tool='verlator', tool='verilator=/path/to/verilator.exe'
 
-    Performs no update if tool has no = in it. Returns tool (str) w/out = in it
+    Performs no update if tool has no = or : in it. Returns tool (str) w/out = in it
     '''
-    if not tool or '=' not in tool:
-        return tool
-
-    tool, user_exe = tool.split('=')[0:2]
-
-    user_exe = shutil.which(user_exe)
-
-    # try adding to $PATH if in form --tool=/path/to/exe
-    tool_try_add_to_path(tool)
-
-    if tool not in config['auto_tools_order'][0]:
-        return tool
-    if not user_exe:
-        return tool
-
-    old_exe = config['auto_tools_order'][0][tool].get('exe', str())
-    if isinstance(old_exe, list):
-        config['auto_tools_order'][0][tool]['exe'][0] = user_exe
-    else:
-        config['auto_tools_order'][0][tool]['exe'] = user_exe
-    return tool
+    return tool_try_add_to_path(tool=tool, config=config, update_config=True)
 
 
 def update_config_auto_tool_order_for_tools(tools: list, config: dict) -> list:
@@ -247,7 +232,7 @@ def get_config_merged_with_defaults(config:dict) -> dict:
 def get_argparser() -> argparse.ArgumentParser:
     '''Returns an ArgumentParser, handles --config-yml=<filename> arg'''
     parser = argparse.ArgumentParser(
-        prog='opencos eda config options', add_help=False, allow_abbrev=False
+        prog=f'{safe_emoji("🔎 ")}opencos eda config options', add_help=False, allow_abbrev=False
     )
     parser.add_argument('--config-yml', type=str, default=Defaults.config_yml,
                         help=('YAML filename to use for configuration (default'
@@ -327,32 +312,92 @@ def write_eda_config_and_args(
     yaml_safe_writer(data=data, filepath=fullpath)
 
 
-def tool_try_add_to_path(tool: str) -> None:
-    '''Since we support --tool=<name>=/path/to/bin/exe, attempt to prepend $PATH
+def tool_arg_get_parts(tool: str) -> list:
+    '''Given a tool (str or None) that may be in form <name>=/path/to/something
+
+    Return the parts [<name>, <path>, ..]
+    '''
+    if not tool or ('=' not in tool and ':' not in tool):
+        return [tool]
+
+    if '=' in tool:
+        parts = tool.split('=')
+    else:
+        parts = tool.split(':')
+
+    return parts
+
+def tool_arg_remove_path_information(tool: str) -> str:
+    '''Given a tool (str or None) that may be in form <name>=/path/to/something
+
+    Return the <name> only
+    '''
+    if not tool:
+        return tool
+    return tool_arg_get_parts(tool)[0]
+
+
+def tool_try_add_to_path( # pylint: disable=too-many-branches
+        tool: str, config: dict, update_config: bool
+) -> str:
+    '''Since we support --tool=<name>=/path/to/bin[/exe], attempt to prepend $PATH
+
+    (also works for --tool=<name>:/path/to/bin[/exe] )
 
     with this information for this tool (which will nicely affect all subprocesses,
     but not wreck our original shell).'''
 
-    if not tool or '=' not in tool:
-        return
+    name_path_parts = tool_arg_get_parts(tool)
+    if len(name_path_parts) == 1:
+        return name_path_parts[0]
 
-    name, exe = tool.split('=')
-    if os.path.isdir(name):
-        # Someone passes us --tool=<name>=/path/to/bin/ (did not have exe)
-        path = name
+    name, path_arg = name_path_parts[0:2]
+
+    if name not in config['auto_tools_order'][0]:
+        return name
+
+    config_exe = config['auto_tools_order'][0][name].get('exe', str())
+    if isinstance(config_exe, list):
+        orig_exe = config_exe[0]
     else:
-        # Someone passes us --tool=<name>=/path/to/bin/exe, remove the exe.
-        path, _ = os.path.split(exe)
-    if not path:
-        return
+        orig_exe = config_exe
+
+    if path_arg and os.path.isfile(path_arg):
+        # Someone passes us --tool=<name>=/path/to/bin/exe, remove the exe from path:
+        path, exe = os.path.split(path_arg)
+    elif path_arg and os.path.isdir(path_arg):
+        # Someone passes us --tool=<name>=/path/to/bin/ (did not have exe)
+        path, exe = path_arg, orig_exe
+    else:
+        path, exe = '', ''
+
+    if not path or not exe:
+        util.error(f'Can not find path or exe for --tool={tool}: {name=} path={path_arg}')
+        return name
 
     path = os.path.abspath(path)
     if os.path.isdir(path):
-        paths = os.environ['PATH'].split(':')
+        paths = os.environ.get('PATH', '').split(':')
         if path not in paths:
-            os.environ['PATH'] = path + ':' + os.environ['PATH']
             util.info(f'--tool={tool} has path information, prepending PATH with: {path}')
+            os.environ['PATH'] = path + ':' + os.environ.get('PATH', '')
         else:
             util.info(f'--tool={tool} has path information, but {path} already in $PATH')
-    if exe and os.path.isfile(exe):
-        util.info(f'--tool={tool} has path information, using exe {shutil.which(exe)}')
+
+    user_exe = os.path.join(path, exe)
+    if not os.access(user_exe, os.X_OK):
+        util.error(f'--tool setting for {tool}: {user_exe} is not an executable')
+        return name
+
+    user_exe = shutil.which(exe)
+
+    if update_config:
+        if isinstance(config_exe, list):
+            config['auto_tools_order'][0][name]['exe'][0] = user_exe
+        else:
+            config['auto_tools_order'][0][name]['exe'] = user_exe
+        util.debug(f'For {tool=}, auto_tools_order config updated')
+
+    util.debug(f'For {tool=}, final {user_exe=}')
+
+    return name
