@@ -42,12 +42,18 @@ def json_paths_to_jsonl(
     with open(output_json_path, 'w', encoding='utf-8') as outf:
 
         # jsonl is every line of the file is a json.
+        # We would expect the JSON to have "tests": [ ... ] with >= 1 test.
         for json_file_path in json_file_paths:
             with open(json_file_path, encoding='utf-8') as f:
                 data = json.load(f)
                 if len(assert_json_types) > 0 and type(data) not in assert_json_types:
                     error(f'{json_file_path=} JSON data is not a Table (py dict) {type(data)=}')
-                json.dump(data, outf)
+                if 'tests' in data and isinstance(data['tests'], list):
+                    for test in data['tests']:
+                        json.dump(test, outf)
+                else:
+                    json.dump(data, outf)
+
                 outf.write('\n')
         info(f'Wrote {len(json_file_paths)} tests to {output_json_path=}')
 
@@ -78,7 +84,12 @@ def json_paths_to_single_json(
                 data = json.load(f)
                 if len(assert_json_types) > 0 and type(data) not in assert_json_types:
                     error(f'{json_file_path=} JSON data is not a Table (py dict) {type(data)=}')
-                out_json_data['tests'].append(data)
+                if 'tests' in data and isinstance(data['tests'], list):
+                    for test in data['tests']:
+                        out_json_data['tests'].append(test)
+                else:
+                    out_json_data['tests'].append(data)
+
         json.dump(out_json_data, outf)
         outf.write('\n')
         info(f'Wrote {len(json_file_paths)} tests {output_json_path=}')
@@ -471,7 +482,52 @@ class ExportHelper:
         yaml_safe_writer(data=data, filepath=dst)
 
 
-    def create_export_json_in_out_dir( # pylint: disable=unused-argument
+    def create_combined_env_file_in_out_dir(self) -> list:
+        ''' Returns list of all .env lines to be put in output directory
+
+        Creates single exported .env file if any --env-file(s)
+        were loaded by CLI or DEPS targets. --input-file/-f was handled as args
+        already, but --env-files need to be special cased. We do not add this
+        combined ".env" file to the all_files list, just build it on the fly if needed.
+        '''
+        env_lines = []
+        for filepath in util.env_files_loaded:
+            with open(filepath, encoding='utf-8') as f:
+                env_lines.extend(f.readlines() + ['\n'])
+
+        if env_lines:
+            dst = os.path.join(self.out_dir, '.env')
+
+            # Write this to the export directory too:
+            with open(dst, 'w', encoding='utf-8') as f:
+                for lineno, line in enumerate(env_lines):
+
+                    # modify VERILOG_SOURCES line if present, although it is known via DEPS.yml,
+                    # we can set it according to files_v and files_sv:
+                    if line.strip().startswith('VERILOG_SOURCES'):
+                        base_verilog_filenames = [
+                            os.path.split(x)[1] for x in \
+                            self.cmd_design_obj.files_v + self.cmd_design_obj.files_sv
+                        ]
+                        env_lines[lineno] = (
+                            f'VERILOG_SOURCES = {" ".join(base_verilog_filenames)}'
+                        )
+                        continue
+
+                    # remove PYTHONPATH from .env
+                    if line.strip().startswith('PYTHONPATH'):
+                        env_lines[lineno] = ''
+                        continue
+
+                    f.write(line)
+
+
+            info(f'export_helper: Wrote {dst}')
+
+        return env_lines
+
+
+    def create_export_json_in_out_dir( # pylint: disable=unused-argument,too-many-locals,too-many-branches
             self, eda_config:dict={}, **kwargs
     ) -> None:
         '''Optionally creates an exported JSON file in the output directory'''
@@ -482,40 +538,43 @@ class ExportHelper:
         # assumes we've run self.create_deps_yml_in_out_dir():
         assert self.target
         assert self.out_deps_file
+        full_eda_cmd_list = ['eda', self.eda_command]
+        if self.args.get('waves', False):
+            full_eda_cmd_list.append('--waves')
+        if self.args.get('tool', ''):
+            full_eda_cmd_list.append(f'--tool={self.args["tool"]}')
+        full_eda_cmd_list.append(self.target)
 
         data = {
-            'name': self.target,
-            'eda': {
-                'enable': True,
-                'multi': False, # Not yet implemented.
-                'command': self.eda_command,
-                'targets': [self.target],
-                'args': [],
-                'waves': self.args.get('waves', False),
-                # tool - eda.CommandSimVerilator has this set in self.args:
-                'tool': self.args.get('tool', None),
-            },
-            'files': [],
+            'correlationId': self.target,
+            'jobType': 'edaCmd',
+            'cmd': ' '.join(full_eda_cmd_list),
+            'timeout': 600,
+            'filesList': [], # filename (str), content (str)
         }
 
-        # allow caller to override eda - tool, or eda - args, etc.
-        for k,v in eda_config.items():
-            if k in data['eda'] and v is not None:
-                data['eda'][k] = v
-
         # Note that args may already be set via:
         #   create_deps_yml_in_out_dir(deps_file_args=some_list)
         # For example, eda.CommandSim.do_export() will set certain allow-listed
         # args if present with non-default values.
 
-
         all_files = [self.out_deps_file] + self.included_files \
             + self.cmd_design_obj.files_sv + self.cmd_design_obj.files_v \
             + self.cmd_design_obj.files_vhd + self.cmd_design_obj.files_cpp \
+            + self.cmd_design_obj.files_sdc + self.cmd_design_obj.files_py \
+            + self.cmd_design_obj.files_makefile + self.cmd_design_obj.files_non_source
+
+        all_files = list(dict.fromkeys(all_files)) # uniqify list.
+
+        # The last file we handle is a single exported .env file if any --env-file(s)
+        env_lines = self.create_combined_env_file_in_out_dir()
+        if env_lines:
+            # write the updated env_lines to the export.json file.
+            data['filesList'].append({
+                'filename': '.env',
+                'content': ''.join(env_lines), # already has \n per line
+            })
 
-        for x in self.cmd_design_obj.files_non_source:
-            if x not in all_files:
-                all_files.append(x)
 
         for somefile in all_files:
 
@@ -532,18 +591,17 @@ class ExportHelper:
 
             assert os.path.exists(somefile)
             with open(somefile, encoding='utf-8') as f:
-                filestr = ''.join(f.readlines())
-                data['files'].append({
-                    'name': os.path.split(somefile)[1],
-                    'content': filestr,
+                data['filesList'].append({
+                    'filename': os.path.split(somefile)[1],
+                    'content': ''.join(f.readlines()),
                 })
 
-
+        test_runner_data = {'tests': [data]} # single test for test runner.
         dst = os.path.join(self.out_dir, 'export.json')
         with open(dst, 'w', encoding='utf-8') as f:
-            json.dump(data, f)
+            json.dump(test_runner_data, f)
             f.write('\n')
-        info(f'export_helper: Wrote {dst=}')
+        info(f'export_helper: Wrote {dst}')
 
         # If this was from an `export` command, and the self.out_dir != self.args['work-dir'], then
         # copy the export.json to the work-dir:
