@@ -13,11 +13,12 @@ just needs to set a couple of these statically.
 all values in this file are in hex.
 '''
 
-table = {
+NAMES = {
 
     'OC_VENDOR': {
         0: "None",
         1: "Xilinx",
+        2: "Altera",
     },
 
     'OC_BOARD': {
@@ -28,11 +29,13 @@ table = {
         4: "U55N",
         5: "U50C",
         6: "PYNQ-Z2",
+        7: "AG3C_DEVKIT",
     },
 
     'OC_LIBRARY': {
         0: "None",
         1: "Ultrascale+",
+        2: "Agilex3"
     },
 
     'PLL_TYPES': {
