@@ -20,15 +20,15 @@ and should be more gracefully handled.
 import os
 import shutil
 import subprocess
-from contextlib import redirect_stdout, redirect_stderr
 
 import pytest
 
-from opencos import eda, eda_tool_helper
+from opencos import eda
 from opencos.utils.markup_helpers import yaml_safe_load
 from opencos.tests import helpers
 from opencos.tests.helpers import eda_wrap, eda_sim_wrap, eda_elab_wrap, \
-    Helpers
+    Helpers, tools_loaded, can_run_eda_sim
+
 
 
 THISPATH = os.path.dirname(__file__)
@@ -37,17 +37,6 @@ def chdir_remove_work_dir(relpath):
     '''Changes dir to relpath, removes the work directories (eda.work, eda.export*)'''
     return helpers.chdir_remove_work_dir(THISPATH, relpath)
 
-# Figure out what tools the system has available, without calling eda.main(..)
-config, tools_loaded = eda_tool_helper.get_config_and_tools_loaded()
-
-def can_run_eda_sim() -> bool:
-    '''Returns True if we have any installed tool that can run: eda sim'''
-    return helpers.can_run_eda_command('sim', config=config)
-
-def can_run_eda_elab() -> bool:
-    '''Returns True if we have any installed tool that can run: eda elab'''
-    return helpers.can_run_eda_command('elab', config=config)
-
 @pytest.mark.skipif(
     'verilator' not in tools_loaded and 'vivado' not in tools_loaded,
     reason="requires verilator OR vivado"
@@ -107,12 +96,23 @@ class TestsRequiresVerilator( # pylint: disable=too-many-public-methods
         assert rc == 0
 
 
+
     def test_args_sim_tool_with_path(self):
         '''Test for calling a tool as --tool=<tool>=</path/to/tool-exe>'''
         verilator_fullpath = shutil.which('verilator')
+        verilator_path, _ = os.path.split(verilator_fullpath)
+
         chdir_remove_work_dir('../../lib/tests')
         rc = eda_sim_wrap('--tool', f'verilator={verilator_fullpath}', 'oclib_fifo_test')
-        print(f'{rc=}')
+        assert rc == 0
+
+        rc = eda_sim_wrap('--tool', f'verilator:{verilator_fullpath}', 'oclib_fifo_test')
+        assert rc == 0
+
+        rc = eda_sim_wrap('--tool', f'verilator={verilator_path}', 'oclib_fifo_test')
+        assert rc == 0
+
+        rc = eda_sim_wrap('--tool', f'verilator:{verilator_fullpath}', 'oclib_fifo_test')
         assert rc == 0
 
     def test_args_sim_with_coverage(self):
@@ -659,6 +659,31 @@ class TestsRequiresIVerilog(Helpers):
         assert rc == 0
 
 
+@pytest.mark.skipif(not can_run_eda_sim(), reason='no tool found to handle command: sim')
+class TestArgs(Helpers):
+    '''Test some args features, needs a sim tool'''
+    DEFAULT_DIR = os.path.join(THISPATH, '..', '..', 'lib', 'tests')
+
+    def test_duplicate_args(self):
+        '''Use oclib_fifo_test to make sure we don't lose (do NOT uniquify) duplicate
+        list-style args'''
+        self.chdir()
+        rc = self.log_it(
+            'sim --stop-before-compile oclib_fifo_test --compile-args=-hi --compile-args=-hi',
+            use_eda_wrap=False
+            )
+        assert rc == 0
+        # Confirm we have two args in self.args['compile-args'] for: -hi
+        eda_config_yml_path = os.path.join(
+            os.getcwd(), 'eda.work', 'oclib_fifo_test.sim', 'eda_output_config.yml'
+        )
+        data = yaml_safe_load(eda_config_yml_path)
+        assert 'args' in data
+        assert 'compile-args' in data['args']
+        assert len(data['args']['compile-args']) == 2
+        assert data['args']['compile-args'] == ['-hi', '-hi']
+
+
 @pytest.mark.skipif(not can_run_eda_sim(), reason='no tool found to handle command: sim')
 class TestDepsReqs:
     '''Tests for 'reqs' in the DEPS files. 'reqs' are requirements, like a .pcap or file
@@ -708,65 +733,6 @@ class TestDepsReqs:
         assert rc > 1
 
 
-@pytest.mark.parametrize("command", ['sim', 'shell'])
-def test_deps_command_order(command):
-    '''Test for various "commands" within a DEPS target. This test checks that command
-    order is preserved in the top-to-bottom deps order, meaning that eda.py has to collect
-    all commands deps order, and then execute them in that order.'''
-
-    chdir_remove_work_dir('deps_files/command_order')
-    if command == 'sim' and not can_run_eda_sim():
-        pytest.skip(f'sim skipped, {can_run_eda_sim()=}')
-        return # skip/pass
-
-    if command == 'shell':
-        cmd_list = 'shell target_test'.split()
-    else:
-        cmd_list = 'sim --stop-before-compile target_test'.split()
-
-    with open('eda.log', 'w', encoding='utf-8') as f:
-        with redirect_stdout(f):
-            with redirect_stderr(f):
-                rc = eda.main(*cmd_list)
-
-    print(f'{rc=}')
-    assert rc == 0
-
-    # We should see "hi" before "bye" to confirm deps + command order is correct.
-    # see ./deps_files/command_order/DEPS.yml - target = target_test
-    found_str_list = [
-        'exec: echo "hi"',
-        'exec: echo "bye"',
-    ]
-    found_lines_list = [None, None]
-
-    with open('eda.log', encoding='utf-8') as f:
-        for lineno, line in enumerate(f.readlines()):
-            line = line.rstrip()
-            for idx,key in enumerate(found_str_list):
-                if key in line:
-                    found_lines_list[idx] = lineno
-
-    assert found_lines_list[0] # found hi
-    assert found_lines_list[1] # found bye
-    assert found_lines_list[0] < found_lines_list[1] # hi before bye
-
-    # Added check, we redirected to create eda.log earlier to confirm the targets worked,
-    # but as a general eda.py check, all shell commands should create their own
-    # {target}__shell_0.log file:
-    work_dir = os.path.join(
-        THISPATH, 'deps_files', 'command_order', 'eda.work', f'target_test.{command}'
-    )
-    # Note that eda will write out the returncode INFO line to tee'd log files, so
-    # there is more in the log file than "hi" or "bye".
-    with open(os.path.join(work_dir, 'target_echo_hi__shell_0.log'), encoding='utf-8') as f:
-        text = ' '.join(f.readlines()).strip()
-        assert any(text.startswith(x) for x in ['hi', '"hi"', '\\"hi\\"'])
-    # Added check, one of the targets uses a custom 'tee' file name, instead of the default log.
-    with open(os.path.join(work_dir, 'custom_tee_echo_bye.log'), encoding='utf-8') as f:
-        text = ''.join(f.readlines()).strip()
-        assert any(text.startswith(x) for x in ['bye', '"bye"', '\\"bye\\"'])
-
 
 @pytest.mark.skipif('verilator' not in tools_loaded, reason="requires verilator")
 class TestDepsOtherMarkup:
