@@ -10,7 +10,9 @@ import subprocess
 from opencos import util
 from opencos.eda_base import Tool
 from opencos.commands import CommandSim
+from opencos.utils import status_constants
 from opencos.utils.str_helpers import sanitize_defines_for_sh
+from opencos.tools import verilator # For default waivers.
 
 
 class ToolCocotb(Tool):
@@ -76,6 +78,8 @@ class CommandSimCocotb(CommandSim, ToolCocotb):
             'tcl-file': None,
             'cocotb-test-module': None,
             'cocotb-test-runner': 'python',
+            'cocotb-test-runner-file': '',
+            'cocotb-test-run-dir': None, # If None, use self.args['work-dir']
             'cocotb-simulator': 'verilator',
             'cocotb-makefile': False,
             'cocotb-python-runner': True,
@@ -86,17 +90,25 @@ class CommandSimCocotb(CommandSim, ToolCocotb):
             'waves': 'Include waveforms by setting COCOTB_ENABLE_WAVES=1',
             'cocotb-test-module': 'Python test module name (e.g., test_my_design)',
             'cocotb-test-runner': 'Test runner to use: python (default) or pytest',
-            'cocotb-simulator': ('Simulator backend: verilator (default), icarus, etc.'
-                                 ' Note that iverilog will convert to icarus here'),
+            'cocotb-test-runner-file': 'Bring your own test_runner.py file',
+            'cocotb-test-run-dir': (
+                'Directory where cocotb-test-runner will run, if unset uses default eda.work/(name)'
+            ),
+            'cocotb-simulator': (
+                'Simulator backend: verilator (default), icarus, etc.'
+                ' Note that iverilog will convert to icarus here'
+            ),
             'cocotb-makefile': 'Use traditional Makefile system instead of Python runner',
             'cocotb-python-runner': 'Use Python-based runner system (default, cocotb 1.8+)',
-            'cocotb-standalone-makefile': ('Use provided Makefile as-is, '
-                                          'run make in source directory'),
+            'cocotb-standalone-makefile': (
+                'Use provided Makefile as-is, run make in source directory'
+            ),
         })
 
         self.cocotb_command_lists = []
         self.cocotb_test_files = []
 
+
     def set_tool_defines(self):
         ToolCocotb.set_tool_defines(self)
 
@@ -155,12 +167,13 @@ class CommandSimCocotb(CommandSim, ToolCocotb):
             simulate_sh_fname='cocotb_test.sh'
         )
 
+
     def _find_cocotb_test_files(self):
         '''Find Python test files that contain cocotb tests'''
         self.cocotb_test_files = []
 
         # Look for test files in the current directory and deps
-        for file_path in self.files_non_source:
+        for file_path in self.files_non_source + self.files_py:
             if (file_path.endswith('.py') and
                 ('test' in file_path.lower() or 'tb' in file_path.lower())):
                 # Check if it's a cocotb test file
@@ -193,8 +206,19 @@ class CommandSimCocotb(CommandSim, ToolCocotb):
     def _prepare_python_runner_system(self):
         '''Prepare cocotb using the Python-based runner system (cocotb 1.8+)'''
 
-        # Create a Python runner script
-        runner_script = self._create_python_runner_script()
+        # Create a Python runner script, or use from --cocotb-test-runner-file
+        if self.args['cocotb-test-runner-file']:
+
+            runner_script = self.args['cocotb-test-runner-file']
+            if not os.path.isfile(self.args['cocotb-test-runner-file']):
+                self.error(
+                    "File does not exist, for: --cocotb-test-runner-file=",
+                    f"{self.args['cocotb-test-runner-file']}",
+                    error_code=status_constants.EDA_GENERAL_FILE_NOT_FOUND
+                )
+                return
+        else:
+            runner_script = self._create_python_runner_script()
 
         if self.args['cocotb-test-runner'] == 'pytest':
             # Use pytest to run the tests
@@ -212,28 +236,41 @@ class CommandSimCocotb(CommandSim, ToolCocotb):
         # Set environment variables
         env_vars = self._get_cocotb_env_vars()
 
-        # Create command with environment variables
-        command_list = self._create_env_command(env_vars) + cmd_list
-        self.cocotb_command_lists = [util.ShellCommandList(command_list,
-                                                          tee_fpath='cocotb_test.log')]
+        # Create command with environment variables, needs to be list-of-(cmd_list)
+        self.cocotb_command_lists = [
+            util.ShellCommandList(
+                self._create_env_command(env_vars) + cmd_list,
+                tee_fpath='cocotb_test.log',
+                # If someone wanted to run this from a specific dir, then
+                # run it from there.
+                work_dir=self.args['cocotb-test-run-dir']
+            )
+        ]
 
     def _prepare_makefile_system(self):
         '''Prepare cocotb using the traditional Makefile system'''
 
         makefile_path = os.path.join(self.args['work-dir'], 'Makefile')
+        self.files_makefile.append(makefile_path)
         with open(makefile_path, 'w', encoding='utf-8') as f:
             f.write(self._create_makefile_content())
 
         cmd_list = self._create_shell_command_with_success('make -f Makefile')
-        self.cocotb_command_lists = [util.ShellCommandList(cmd_list,
-                                                          tee_fpath='cocotb_makefile.log')]
+        self.cocotb_command_lists = [
+            util.ShellCommandList(
+                cmd_list, tee_fpath='cocotb_makefile.log',
+                # If someone wanted to run this from a specific dir, then
+                # run it from there.
+                work_dir=self.args['cocotb-test-run-dir']
+            )
+        ]
 
     def _prepare_standalone_makefile_system(self):
         '''Use provided Makefile as-is, run make in source directory'''
 
         # Find the Makefile in our dependencies
         makefile_path = None
-        for file_path in self.files_non_source:
+        for file_path in self.files_non_source + self.files_makefile:
             if os.path.basename(file_path).lower() == 'makefile':
                 makefile_path = file_path
                 break
@@ -241,10 +278,19 @@ class CommandSimCocotb(CommandSim, ToolCocotb):
         if not makefile_path:
             self.error('No Makefile found in deps for --cocotb-standalone-makefile')
 
+        if any(value for key,value in self.args.items() if key.startswith('export')):
+            util.warning(f'Using an --export arg with Cocotb standalong Makefile {makefile_path}',
+                         'is not fully supported')
+
         makefile_dir = os.path.dirname(os.path.abspath(makefile_path))
-        cmd_list = self._create_shell_command_with_success(f'cd {makefile_dir} && make')
-        self.cocotb_command_lists = [util.ShellCommandList(
-            cmd_list, tee_fpath='cocotb_standalone.log')]
+        cmd_list = self._create_shell_command_with_success('make')
+        self.cocotb_command_lists = [
+            util.ShellCommandList(
+                cmd_list,
+                tee_fpath='cocotb_standalone.log',
+                work_dir=makefile_dir
+            )
+        ]
 
     def _get_test_module_name(self) -> str:
         '''Get the test module name from args or detected files'''
@@ -280,8 +326,22 @@ class CommandSimCocotb(CommandSim, ToolCocotb):
 
         return script_path
 
+
     def _generate_runner_script_content(self, test_module: str, hdl_sources: list) -> str:
         '''Generate the content for the Python runner script'''
+
+        if shutil.which('verilator'):
+            # TODO(drew): this shortcuts if verilator is truly usable,
+            # consider using eda_tool_helper to get "tools_loaded", which
+            # is not set in self.config['tools_loaded'] when --tool=cocotb.
+            # Would need minor refactor for eda.py methods auto_tools_order,
+            # tool_setup to go into eda_tool_helper.py, and those methods would
+            # need hooks to be non-destructive to config.
+            tmp_verilator_obj = verilator.VerilatorSim(config=self.config)
+            verilator_waivers = tmp_verilator_obj.get_verilator_tool_config_waivers()
+        else:
+            verilator_waivers = []
+
         return f'''#!/usr/bin/env python3
 """
 Cocotb test runner script generated by opencos
@@ -337,6 +397,8 @@ def run_cocotb_test():
 
         if simulator == "verilator":
             build_args.extend({list(self.args.get('verilate-args', []))!r})
+            build_args.extend({list(self.args.get('compile-args', []))!r})
+            build_args.extend({list(verilator_waivers)!r})
 
         # Build the design
         runner.build(
@@ -449,9 +511,20 @@ include $(shell cocotb-config --makefiles)/Makefile.sim
         '''Get environment variables for cocotb execution'''
         env_vars = {}
 
+        top = self.args.get('top', 'top')
+
         # Basic cocotb configuration
-        env_vars['SIM'] = self.args['cocotb-simulator']
-        env_vars['TOPLEVEL'] = self.args.get('top', 'top')
+        env_vars.update({
+            'SIM': self.args['cocotb-simulator'],
+            'TOPLEVEL': top,
+            'COCOTB_TOPLEVEL': top,
+        })
+
+        if test_module := self.args['cocotb-test-module']:
+            env_vars.update({
+                'MODULE': test_module,
+                'COCOTB_TEST_MODULES': test_module,
+            })
 
         # Enable waves if requested
         if self.args.get('waves', False):
@@ -464,8 +537,8 @@ include $(shell cocotb-config --makefiles)/Makefile.sim
             env_vars['COCOTB_LOG_LEVEL'] = 'INFO'
 
         # Random seed
-        if self.args.get('seed'):
-            env_vars['COCOTB_RANDOM_SEED'] = str(self.args['seed'])
+        if seed := self.args.get('seed'):
+            env_vars['COCOTB_RANDOM_SEED'] = str(seed)
 
         return env_vars
 
