@@ -160,6 +160,10 @@ class CommandSimIverilog(CommandSim, ToolIverilog):
         cmd_list += self.tool_config.get('simulate-args', '').split()
         if self.args['waves']:
             cmd_list += self.tool_config.get('simulate-waves-args', '').split()
+        for x in self.args['sim-plusargs']:
+            if x[0] != '+':
+                x = f'+{x}'
+            cmd_list.append(x)
         return [ util.ShellCommandList(cmd_list, tee_fpath='sim.log') ]
 
     def get_post_simulate_command_lists(self, **kwargs) -> list:
