@@ -32,6 +32,14 @@ class CommandSimModelsimAse(CommandSim, ToolModelsimAse):
         self.args.update({
             'tool': self._TOOL, # override
             'gui': False,
+            'vopt': self.use_vopt,
+        })
+        self.args_help.update({
+            'vopt': (
+                'Boolean to enable/disable use of vopt step prior to vsim step'
+                ' Note that vopt args can be controlled with --elab-args=<value1>'
+                ' --elab-args=<value2> ...'
+            )
         })
 
     def set_tool_defines(self):
@@ -158,7 +166,7 @@ class CommandSimModelsimAse(CommandSim, ToolModelsimAse):
                 'compile-waivers',
                 [ #defaults:
                     '2275', # 2275 - Existing package 'foo_pkg' will be overwritten.
-                ]):
+                ]) + self.args['compile-waivers']:
             vlog_dot_f_lines += ['-suppress', str(waiver)]
 
         if self.args['gui'] or self.args['waves']:
@@ -252,15 +260,18 @@ class CommandSimModelsimAse(CommandSim, ToolModelsimAse):
 
         sim_plusargs_str = self._get_sim_plusargs_str()
         vsim_suppress_list_str = self._get_vsim_suppress_list_str()
+        vsim_ext_args = ' '.join(self.args.get('sim-args', []))
 
-        voptargs_str = ""
+        voptargs_str = self.tool_config.get('elab-args', '')
+        voptargs_str += ' '.join(self.args.get('elab-args', []))
         if self.args['gui'] or self.args['waves']:
-            voptargs_str = self.tool_config.get('simulate-waves-args', '+acc')
+            voptargs_str += ' ' + self.tool_config.get('simulate-waves-args', '+acc')
             util.artifacts.add_extension(
                 search_paths=self.args['work-dir'], file_extension='wlf',
                 typ='waveform', description='Modelsim/Questa Waveform WLF (Wave Log Format) file'
             )
 
+        # TODO(drew): support self.args['sim_libary'] (1 lists)
         vlog_do_lines = []
         vsim_do_lines = []
 
@@ -269,9 +280,6 @@ class CommandSimModelsimAse(CommandSim, ToolModelsimAse):
         voptargs_str += more_voptargs_str
 
 
-        # TODO(drew): support self.args['sim_libary', 'elab-args', sim-args'] (3 lists)
-        # to add to vsim_one_liner.
-
         vopt_one_liner = ""
         if self.use_vopt:
             vopt_one_liner = (
@@ -283,12 +291,12 @@ class CommandSimModelsimAse(CommandSim, ToolModelsimAse):
 
             vsim_one_liner = "vsim -onfinish stop" \
                 + f" -sv_seed {self.args['seed']} {sim_plusargs_str} {vsim_suppress_list_str}" \
-                + f" opt__{self.args['top']}"
+                + f" {vsim_ext_args} opt__{self.args['top']}"
         else:
             # vopt doesn't exist, use single vsim call after vlog call:
             vsim_one_liner = "vsim -onfinish stop" \
                 + f" -sv_seed {self.args['seed']} {sim_plusargs_str} {vsim_suppress_list_str}" \
-                + f" {voptargs_str} work.{self.args['top']}"
+                + f" {voptargs_str} {vsim_ext_args} work.{self.args['top']}"
 
 
         vsim_one_liner = vsim_one_liner.replace('\n', ' ')
@@ -393,7 +401,7 @@ class CommandSimModelsimAse(CommandSim, ToolModelsimAse):
                     #defaults:
                     '3009', # 3009: [TSCALE] - Module 'foo' does not have a timeunit/timeprecision
                             #       specification in effect, but other modules do.
-                ]):
+                ]) + self.args['sim-waivers']:
             vsim_suppress_list += ['-suppress', str(waiver)]
 
         return ' '.join(vsim_suppress_list)
