@@ -13,6 +13,7 @@ import subprocess
 from opencos import util
 from opencos.tools.modelsim_ase import ToolModelsimAse, CommandSimModelsimAse
 from opencos.utils.str_helpers import sanitize_defines_for_sh
+from opencos.utils import status_constants
 
 class ToolRiviera(ToolModelsimAse):
     '''ToolRiviera used by opencos.eda for --tool=riviera'''
@@ -59,6 +60,22 @@ class CommandSimRiviera(CommandSimModelsimAse, ToolRiviera):
         self.args.update({
             'tool': self._TOOL, # override
             'gui': False,
+            'waves-fst': True,
+            'waves-vcd': False,
+            'coverage-tcl': '',
+        })
+        self.args_help.update({
+            'waves-fst': (
+                '(Default True) If using --waves, apply simulation runtime arg +trace.'
+                ' Note that if you do not have SV code using $dumpfile, eda will add'
+                ' _waves_pkg.sv to handle this for you with +trace runtime plusarg.'
+            ),
+            'waves-vcd': 'If using --waves, apply simulation runtime arg +trace=vcd',
+            'waves': 'Save a .asdb offline wavefile, can be used with --waves-fst or --waves-vcd',
+            'coverage-tcl': (
+                'bring your own .tcl file to run in Riviera (vsim) for coverage. The default'
+                ' tcl steps are (from tool config in --config-yml):   '
+            ) + '; '.join(self.tool_config.get('simulate-coverage-tcl', [])),
         })
 
     def set_tool_defines(self):
@@ -76,6 +93,9 @@ class CommandSimRiviera(CommandSimModelsimAse, ToolRiviera):
 
     def compile(self):
         '''Override for CommandSimModelsimAse.compile() so we can set our own must_strings'''
+
+        self.add_waves_pkg_file()
+
         if self.args['stop-before-compile']:
             # don't run anything, save everyting we've already run in _prep_compile()
             return
@@ -126,7 +146,8 @@ class CommandSimRiviera(CommandSimModelsimAse, ToolRiviera):
             '-sv -input_ports net').split()
 
         # Add waivers from config.tool.riviera, convert to warning:
-        for waiver in self.tool_config.get('compile-waivers', []):
+        for waiver in self.tool_config.get('compile-waivers', []) + \
+                self.args['compile-waivers']:
             vlog_dot_f_lines += [f'-err {waiver} W1']
 
         vlog_dot_f_fname = filename
@@ -167,7 +188,10 @@ class CommandSimRiviera(CommandSimModelsimAse, ToolRiviera):
             f.writelines(line + "\n" for line in vlog_dot_f_lines)
 
 
-    def write_vsim_dot_do(self, dot_do_to_write: list) -> None:
+
+    def write_vsim_dot_do( # pylint: disable=too-many-branches
+            self, dot_do_to_write: list
+    ) -> None:
         '''Writes files(s) based on dot_do_to_write(list of str)
 
         list arg values can be empty (all) or have items 'all', 'sim', 'lint', 'vlog'.'''
@@ -177,9 +201,18 @@ class CommandSimRiviera(CommandSimModelsimAse, ToolRiviera):
         vsim_vlogonly_dot_do_fpath = os.path.join(self.args['work-dir'], 'vsim_vlogonly.do')
 
         sim_plusargs_str = self._get_sim_plusargs_str()
+        vsim_ext_args = ' '.join(self.args.get('sim-args', []))
 
-        voptargs_str = ""
-        if self.args['gui'] or self.args['waves'] or self.args['coverage']:
+        if self.args['waves'] and '+trace' not in sim_plusargs_str:
+            if self.args.get('waves-vcd', False):
+                sim_plusargs_str += ' +trace=vcd'
+            elif self.args.get('waves-fst', False):
+                sim_plusargs_str += ' +trace'
+
+        voptargs_str = self.tool_config.get('elab-args', '')
+        voptargs_str += ' '.join(self.args.get('elab-args', []))
+        if self.args['gui'] or self.args['waves'] or self.args['coverage']: # \
+           #or self.args['waves-asdb']:
             voptargs_str += self.tool_config.get('simulate-waves-args',
                                                  '+accb +accr +access +r+w')
         if self.args['coverage']:
@@ -195,7 +228,7 @@ class CommandSimRiviera(CommandSimModelsimAse, ToolRiviera):
         vsim_one_liner = (
             "vsim"
             f" -sv_seed {self.args['seed']} {sim_plusargs_str}"
-            f" {voptargs_str} work.{self.args['top']}"
+            f" {voptargs_str} {vsim_ext_args} work.{self.args['top']}"
         )
 
         vsim_one_liner = vsim_one_liner.replace('\n', ' ') # needs to be a one-liner
@@ -253,18 +286,21 @@ class CommandSimRiviera(CommandSimModelsimAse, ToolRiviera):
             "}",
         ]
 
+        vsim_dot_do_lines += [
+            "run -all;",
+        ]
         if self.args['coverage']:
-            vsim_dot_do_lines += [
-                "run -all;",
-                "acdb save",
-                "acdb report -db work.acdb -txt -o cov.txt",
-                # Note - could try:
-                ##"cover report -o cov.report.txt -fullverbose -all_columns",
-            ]
-        else:
-            vsim_dot_do_lines += [
-                "run -all;",
-            ]
+            if self.args['coverage-tcl']:
+                tclfile = os.path.abspath(self.args['coverage-tcl'])
+                if not os.path.isfile(tclfile):
+                    self.error(f'--coverage-tcl file not found: {tclfile}',
+                               error_code=status_constants.EDA_COMMAND_OR_ARGS_ERROR)
+                vsim_dot_do_lines += [
+                    f'source {tclfile}'
+                ]
+            else:
+                # default TCL for coverage:
+                vsim_dot_do_lines += self.tool_config.get('simulate-coverage-tcl', [])
 
 
         vsim_dot_do_lines += [
@@ -294,7 +330,7 @@ class CommandSimRiviera(CommandSimModelsimAse, ToolRiviera):
         for waiver in self.tool_config.get(
                 'simulate-waivers', [
                     #defaults: none
-                ]):
+                ]) + self.args['sim-waivers']:
             vsim_suppress_list += ['-filter', str(waiver)]
 
         return ' '.join(vsim_suppress_list)
