@@ -128,6 +128,7 @@ class CommandElabSlang(CommandElab, ToolSlang):
         command_list += self.tool_config.get('compile-args', '--single-unit').split()
         command_list += self.args['slang-args'] # add user args.
         command_list += self._get_slang_json_args(command_exe=command_list[0])
+        command_list += self._get_slang_tool_config_waivers()
 
         # incdirs
         for value in self.incdirs:
@@ -214,6 +215,11 @@ class CommandElabSlang(CommandElab, ToolSlang):
 
         return command_list
 
+    def _get_slang_tool_config_waivers(self) -> list:
+        # Add compile waivers from config and command-line args
+        return [f'-Wno-{waiver}' for waiver in
+                self.tool_config.get('compile-waivers', []) + self.args['compile-waivers']]
+
 
 class CommandLintSlang(CommandElabSlang):
     '''CommandLintSlang is a command handler for: eda lint --tool=slang.'''
