@@ -5,10 +5,12 @@ Contains classes for ToolVerilator and VerilatorSim, VerilatorElab.
 
 # pylint: disable=R0801 # (calling functions with same arguments)
 
+import multiprocessing
 import os
 import shutil
 import subprocess
 
+
 from opencos import util
 from opencos.eda_base import Tool
 from opencos.commands import CommandSim
@@ -76,20 +78,33 @@ class VerilatorSim(CommandSim, ToolVerilator):
             'gui': False,
             'tcl-file': None,
             'dump-vcd': False,
+            'waves-fst': True,
+            'waves-vcd': False,
             'lint-only': False,
             'cc-mode': False,
             'verilator-coverage-args': [],
+            'uvm': False,
             'x-assign': '',
             'x-initial': '',
         })
 
         self.args_help.update({
-            'waves':    ('Include waveforms, if possible for Verilator by applying'
-                         ' simulation runtime arg +trace. User will need SV code to interpret the'
-                         'plusarg and apply $dumpfile("dump.fst").'),
-            'dump-vcd': ('If using --waves, apply simulation runtime arg +trace=vcd. User'
-                         ' will need SV code to interpret the plusarg and apply'
-                         ' $dumpfile("dump.vcd").'),
+            'waves': (
+                'Include waveforms, if possible for Verilator by applying'
+                ' simulation runtime arg +trace. User will need SV code to interpret the'
+                ' plusarg and apply $dumpfile("dump.fst").'
+            ),
+            'waves-fst': (
+                '(Default True) If using --waves, apply simulation runtime arg +trace.'
+                ' Note that if you do not have SV code using $dumpfile, eda will add'
+                ' _waves_pkg.sv to handle this for you with +trace runtime plusarg.'
+            ),
+            'waves-vcd': (
+                'If using --waves, apply simulation runtime arg +trace=vcd. User'
+                ' will need SV code to interpret the plusarg and apply'
+                ' $dumpfile("dump.vcd").'
+            ),
+            'dump-vcd': 'Same as --waves-vcd',
             'lint-only': 'Run verilator with --lint-only, instead of --binary',
             'gui':       'Not supported for Verilator',
             'cc-mode':   'Run verilator with --cc, requires a sim_main.cpp or similar sources',
@@ -102,6 +117,15 @@ class VerilatorSim(CommandSim, ToolVerilator):
                          ' where valid string values are: 0, unique, fast.'
                          ' Also conditinally adds to verilated exe call:'
                          ' +verilator+rand+reset+[0,2] for arg values 0, unique|fast'),
+            'uvm': (
+                'Warns on Verilator < 5.042, or missing $UVM_HOME environment var set (or in'
+                ' .env, $UVM_HOME/uvm_pkg.sv should exist), and will run verilator with args:'
+                ' -Wno-fatal +define+UVM_NO_DPI'
+            ),
+            'verilator-coverage-args': (
+                'Requires --coverage, args to be applied to verilator_coverage, which runs'
+                ' after running the compiled executable simulation'
+            ),
         })
 
         self.verilate_command_lists = []
@@ -124,7 +148,7 @@ class VerilatorSim(CommandSim, ToolVerilator):
         if self.files_cpp:
             self.args['cc-mode'] = True
 
-        self._add_verilator_waves_files()
+        self.add_waves_pkg_file()
 
         # Each of these should be a list of util.ShellCommandList()
         self.verilate_command_lists = self.get_compile_command_lists()
@@ -198,18 +222,19 @@ class VerilatorSim(CommandSim, ToolVerilator):
 
         verilate_command_list = self._get_start_verilator_command_list(lint_only=lint_only)
 
-        # Add compile args from our self.config (tools.verilator.compile-args str)
-        config_compile_args = self.tool_config.get(
-            'compile-args',
-            '--timing --assert --autoflush -sv').split()
-        verilate_command_list += config_compile_args
+        # Handle UVM things (return args), but also handles uvm_pkg.sv in self.files_sv:
+        # since we run this 2x (lint-only and normal) only do warnings for one of them:
+        verilate_command_list += self._verilator_args_uvm(
+            warnings=(not lint_only), add_uvm_pkg_if_found=True
+        )
 
-        verilate_command_list += self._get_verilator_tool_config_waivers()
+        verilate_command_list += self.get_verilator_tool_config_waivers()
 
-        verilate_command_list += self._verilator_args_deal_with_cflags()
+        verilate_command_list += self._verilator_args_defaults_cflags_nproc()
 
         verilate_command_list += self._get_verilator_waves_args(lint_only=lint_only)
 
+
         if self.args.get('coverage', True):
             verilate_command_list += self.tool_config.get(
                 'compile-coverage-args', '--coverage').split()
@@ -300,9 +325,10 @@ class VerilatorSim(CommandSim, ToolVerilator):
                 # Built-in support for eda args --waves and/or --dump-vcd to become runtime
                 # plusargs +trace or +trace=vcd, if +trace or +trace= was not already in our
                 # plusargs.
-                if self.args.get('dump-vcd', False):
+                if self.args.get('dump-vcd', False) or \
+                   self.args.get('waves-vcd', False):
                     sim_plusargs.append('+trace=vcd')
-                else:
+                elif self.args.get('waves-fst', False):
                     sim_plusargs.append('+trace')
 
         verilated_exec_command_list += config_sim_args + sim_plusargs + self.args['sim-args']
@@ -351,40 +377,17 @@ class VerilatorSim(CommandSim, ToolVerilator):
 
         return []
 
-    def _add_verilator_waves_files(self) -> None:
+    def add_waves_pkg_file(self) -> None:
         '''If --waves present, and the user is missing any $dumpfile(), then adds a pre-written
         SystemVerilog package to their source code.
         '''
-        if not self.args['waves']:
-            return
         if self.args['cc-mode']:
-            # We won't do this if the user bround a .cpp file and verilator not
+            # We won't do this if the user brought a .cpp file and verilator not
             # called with --binary.
             return
-        found_dumpfile = False
-        for fname in self.files_v + self.files_sv:
-            if found_dumpfile:
-                break
-            with open(fname, encoding='utf-8') as f:
-                for line in f.readlines():
-                    if '$dumpfile' in line:
-                        found_dumpfile = True
-                        break
-        if not found_dumpfile:
-            thispath = os.path.dirname(__file__)
-            file_to_add = os.path.join(thispath, '..', '_waves_pkg.sv')
-            util.info(f'--waves arg present, no $dumpfile found, adding SV file: {file_to_add}')
-            self.add_file(file_to_add)
 
-        # register .vcd or .fst artifacts:
-        util.artifacts.add_extension(
-            search_paths=self.args['work-dir'], file_extension='fst',
-            typ='waveform', description='Simulation Waveform FST (Fast Signal Trace) file'
-        )
-        util.artifacts.add_extension(
-            search_paths=self.args['work-dir'], file_extension='vcd',
-            typ='waveform', description='Simulation Waveform VCD (Value Change Dump) file'
-        )
+        super().add_waves_pkg_file() # call from CommandSim
+
 
 
     def _get_start_verilator_command_list(self, lint_only: bool = False) -> list:
@@ -405,7 +408,10 @@ class VerilatorSim(CommandSim, ToolVerilator):
 
         return verilate_command_list
 
-    def _get_verilator_tool_config_waivers(self) -> list:
+    def get_verilator_tool_config_waivers(self) -> list:
+        '''Returns list of args to verilator for waviers, from --compile-waivers and
+
+        --config-yml for tool: (config)'''
 
         # Add compile waivers from self.config (tools.verilator.compile-waivers list):
         # list(set(mylist)) to get unique.
@@ -415,7 +421,7 @@ class VerilatorSim(CommandSim, ToolVerilator):
                 [ #defaults:
                     'CASEINCOMPLETE',
                     'TIMESCALEMOD', # If one file has `timescale, then they all must
-                ]):
+                ]) + self.args['compile-waivers']:
             ret.append(f'-Wno-{waiver}')
         return ret
 
@@ -435,45 +441,51 @@ class VerilatorSim(CommandSim, ToolVerilator):
         return ret
 
 
-    def _verilator_args_deal_with_cflags(self) -> list:
+    def _verilator_args_defaults_cflags_nproc(self) -> list:
         '''Returns list of args to be added to verilator (compile) step
 
-        Uses self.args['verilate-args'] and self.args['compile-args']
+        Uses self.args['verilate-args'], self.args['compile-args'], and self.tool_config
+
+        Sets -j <value> and -CFLAGS -O<value> if not present in --config-yml, --compile-args,
+        or --verilate-args. If present, chooses the first instance (does not present duplicates
+        to verilator call).
         '''
 
-        # We can only support one -CFLAGS followed by one -O[0-9] arg in self.args['verilate-args']:
-        verilate_cflags_args_dict = {}
-        verilate_args = [] # will be combined verilate_args + compile-args
-        prev_arg_is_cflags = False
+        # We can only support one -CFLAGS followed by one -O[0-9] arg in
+        # --verilate-args or --compile-args.
+
+        # Add compile args from our self.config (tools.verilator.compile-args str)
+        verilate_args = self.args['verilate-args'] + \
+            self.args['compile-args'] + \
+            self.tool_config.get(
+                'compile-args',
+                '--timing --assert --autoflush -sv').split()
+
         util.debug(f"{self.args['verilate-args']=}")
         util.debug(f"{self.args['compile-args']=}")
-        for arg in self.args['verilate-args'] + self.args['compile-args']:
-            # pick the first ones we see of these:
-            if arg == '-CFLAGS':
-                prev_arg_is_cflags = True
-                if arg not in verilate_cflags_args_dict:
-                    # We can only have 1
-                    verilate_cflags_args_dict[arg] = True
-                    verilate_args.append(arg)
-                else:
-                    util.debug(f'Previous saw -CFLAGS args {verilate_cflags_args_dict=},',
-                               f'skipping new {arg=}')
 
-            elif arg.startswith('-O') and len(arg) == 3:
-                if '-O' not in verilate_cflags_args_dict and prev_arg_is_cflags:
-                    # We can only have 1
-                    verilate_cflags_args_dict['-O'] = arg[-1]
-                    verilate_args.append(arg)
-                else:
-                    util.debug(f'Previous saw -CFLAGS args {verilate_cflags_args_dict=},',
-                               f'skipping new {arg=}')
-                prev_arg_is_cflags = False
+        dash_j_arg_indices = []
+        cflags_dasho_args_indices = []
+        for i, arg in enumerate(list(verilate_args)):
+            # There can only be one of these: -j <value>, similarly can only be one of
+            # -CFLAGS -O<value>
+            if (i + 1) < len(verilate_args):
+                if arg == '-j':
+                    dash_j_arg_indices.extend([i, i + 1])
+                if arg == '-CFLAGS':
+                    next_arg = verilate_args[i + 1]
+                    if next_arg.startswith('-O') and len(next_arg) == 3:
+                        cflags_dasho_args_indices.extend([i, i + 1])
 
-            else:
-                prev_arg_is_cflags = False
-                verilate_args.append(arg)
+        # For -j <value> we'll pick the first one, remove the rest.
+        # Same goes for -CFLAGS -O<value>
+        for index in dash_j_arg_indices[2:] + cflags_dasho_args_indices[2:]:
+            verilate_args[index] = ''
 
-        if '-CFLAGS' in verilate_args:
+        verilate_args = [x for x in verilate_args if x != ''] # strip empty str.
+
+        # Support for --optimize which will use -CFLAGS -O3, if -CFLAGS is not present at all.
+        if cflags_dasho_args_indices:
             # add whatever args were passed via 'compile-args' or 'verilate_args'. Note these will
             # take precedence over the --optimize arg.
             pass
@@ -482,11 +494,87 @@ class VerilatorSim(CommandSim, ToolVerilator):
             # (slower compile, better runtime)
             verilate_args += '-CFLAGS', '-O3'
         else:
+            # Default to -O1:
             verilate_args += '-CFLAGS', '-O1'
 
+        # If there was no -j setting, then use max(2, $(nproc) - 1)
+        if not dash_j_arg_indices:
+            nproc = max(2, multiprocessing.cpu_count() - 1)
+            verilate_args += '-j', f'{nproc}'
+
+
         return verilate_args
 
 
+    def _verilator_support_uvm_pkg_fpath(self, add_if_found: bool = True) -> bool:
+        '''Returns False if we could not find a suitable uvm_pkg.sv to use, or if --no-uvm.
+
+        This will also auto-add uvm_pkg.sv from $UVM_HOME/uvm_pkg.sv if not present in
+        self.files already (adds to front of self.files_sv)
+        '''
+
+        if not self.args['uvm']:
+            return False
+
+        for fname, exists in self.files.items():
+            if exists and os.path.split(fname)[1] == 'uvm_pkg.sv':
+                # already present in our source files (assume someone doing it manually
+                # or via DEPS)
+                return True
+
+        uvm_home = os.environ.get('UVM_HOME', '')
+        if not uvm_home:
+            return False
+
+        uvm_pkg_fpath = os.path.join(uvm_home, 'uvm_pkg.sv')
+        if add_if_found and os.path.isfile(uvm_pkg_fpath):
+            uvm_pkg_fpath = os.path.abspath(uvm_pkg_fpath)
+            util.info(f'For --uvm, adding to source files: {uvm_pkg_fpath}')
+            self.files[uvm_pkg_fpath] = True
+            self.files_sv.insert(0, uvm_pkg_fpath)
+            self.files_caller_info[uvm_pkg_fpath] = 'verilator.py'
+            util.info(f'For --uvm, adding +incdir+: {uvm_home}')
+            self.incdirs.append(os.path.abspath(uvm_home))
+            return True
+
+        return False
+
+
+    def _verilator_args_uvm(
+            self, warnings: bool = True, add_uvm_pkg_if_found: bool = True
+    ) -> list:
+        '''Returns list of args to be added to verilator (compile) step if --uvm present
+
+        Warnings on potential issues (Veriltor version, missing uvm_pkg.sv).
+        Optionally adds uvm_pkg.sv to source files.
+        '''
+
+        # Handle --uvm args:
+        if not self.args['uvm']:
+            return []
+
+        if warnings:
+
+            # prefers Verilator >= v5.042, $UVM_HOME to be set, or warning.
+            version_list = self._VERSION.split('.')
+            if int(version_list[0]) < 5 or \
+               (int(version_list[0]) == 5 and int(version_list[1]) < 42):
+                util.warning(f'Verilator version is {self._VERSION}, --uvm set prefers Verilator',
+                             'version > v5.042')
+
+            if not os.environ.get('UVM_HOME', ''):
+                util.warning('--uvm set, however env (or .env or --env-file) $UVM_HOME is not set')
+
+        uvm_pkg_found = self._verilator_support_uvm_pkg_fpath(add_if_found=add_uvm_pkg_if_found)
+        if warnings and not uvm_pkg_found:
+            util.warning(
+                '--uvm set, however no suitable uvm_pkg.sv is source files,',
+                f'nor in $UVM_HOME/uvm_pkg.sv. $UVM_HOME={os.environ.get("UVM_HOME", "")}'
+            )
+
+        return ['-Wno-fatal', '+define+UVM_NO_DPI']
+
+
     def artifacts_add(self, name: str, typ: str, description: str) -> None:
         '''Override from Command.artifacts_add, so we can catch known file
 
