@@ -536,7 +536,6 @@ class CommandLecYosys(CommandLec, ToolYosys):
 
         self.synth_designs_tops = [None, None]
         self.synth_designs_fpaths = [None, None]
-        self.synth_design_top_module_names = [None, None]
 
     def get_synth_result_fpath(self, target: str) -> str:
         '''Overridden from CommandLec'''
@@ -547,7 +546,7 @@ class CommandLecYosys(CommandLec, ToolYosys):
     def get_synth_command_list(self, design_num: int) -> list:
         '''Returns one of the synthesis command lists, for design_num=0 or 1'''
 
-        if not design_num in [0, 1]:
+        if not design_num in (0, 1):
             self.error(f'{design_num=} we only support LEC on designs 0 and 1')
 
         synth_cmd_list = [
@@ -561,31 +560,43 @@ class CommandLecYosys(CommandLec, ToolYosys):
         synth_cmd_list += [
             '--work-dir=' + self.synth_work_dirs[design_num],
             self.args['designs'][design_num],
-            f'--rename-module=Design{design_num + 1}'
+            f'--rename-module=Design{design_num + 1}',
         ]
 
-        self.synth_design_top_module_names[design_num] = f'Design{design_num + 1}'
+        if self.args['flatten-all']:
+            # We have to do this or may get conflicts on black-boxed modules, but can
+            # be avoided with --no-flatten-all.
+            synth_cmd_list.append(
+                '--flatten-all'
+            )
+
+        self.synth_designs_tops[design_num] = f'Design{design_num + 1}'
 
         return synth_cmd_list
 
 
-    def get_synth_top_from_output_config(self, design_num: int) -> str:
-        '''Returns the top name given the design number that we synthesized'''
+    def get_synth_top_from_output_config(self, design_num: int) -> (str, str):
+        '''Returns the (orignal top name, module name) tuple given the design number
+
+        that we synthesized'''
 
         work_dir = self.synth_work_dirs[design_num]
         output_cfg_fpath = os.path.join(work_dir, eda_config.EDA_OUTPUT_CONFIG_FNAME)
         data = yaml_safe_load(output_cfg_fpath)
         top = data.get('args', {}).get('top', '')
-        if not top:
+        rename_module = data.get('args', {}).get('rename-module', '')
+        if not top and not rename_module:
             self.error(f'"top" not found in synth run from {work_dir=} in',
                        f'config {output_cfg_fpath}')
-        return top
+        if not rename_module:
+            return top, top
+        return top, rename_module
 
 
     def get_synth_results_fpath(self, design_num: int, top: str) -> str:
-        '''Returns the synthesized .v file fpath'''
+        '''Returns the synthesized .v file fpath, using orignal top (not renamed)'''
         if not top:
-            top = self.get_synth_top_from_output_config(design_num=design_num)
+            top, _ = self.get_synth_top_from_output_config(design_num=design_num)
 
         work_dir = self.synth_work_dirs[design_num]
         fpath = os.path.join(work_dir, 'yosys', f'{top}.v')
@@ -594,7 +605,7 @@ class CommandLecYosys(CommandLec, ToolYosys):
         return fpath
 
 
-    def do_it(self) -> None:
+    def do_it(self) -> None: # pylint: disable=too-many-locals,too-many-statements,too-many-branches
         self.set_tool_defines()
         self.write_eda_config_and_args()
 
@@ -622,6 +633,9 @@ class CommandLecYosys(CommandLec, ToolYosys):
                     self.exec(work_dir=self.args['work-dir'], command_list=x,
                               tee_fpath=x.tee_fpath)
 
+            util.info(f'LEC ran via --yosys-scriptfile: {self.args["yosys-scriptfile"]}')
+            return
+
 
         if self.args['synth']:
             synth1_cmd_list = self.get_synth_command_list(design_num=0)
@@ -636,23 +650,23 @@ class CommandLecYosys(CommandLec, ToolYosys):
             self.exec(pwd, synth2_cmd_list, background=True)
             util.info(f'Finished with 2nd LEC synthesis {self.args["designs"][1]}')
 
-            self.synth_designs_tops = [
-                self.get_synth_top_from_output_config(design_num=0),
-                self.get_synth_top_from_output_config(design_num=1)
-            ]
+
+            top0, module0 = self.get_synth_top_from_output_config(design_num=0)
+            top1, module1 = self.get_synth_top_from_output_config(design_num=1)
+            self.synth_designs_tops = [module0, module1]
             util.info(f'Design tops: {self.synth_designs_tops}')
 
             # read the output config
             self.synth_designs_fpaths = [
                 os.path.abspath(
-                    self.get_synth_results_fpath(design_num=0, top=self.synth_designs_tops[0])),
+                    self.get_synth_results_fpath(design_num=0, top=top0)),
                 os.path.abspath(
-                    self.get_synth_results_fpath(design_num=1, top=self.synth_designs_tops[1]))
+                    self.get_synth_results_fpath(design_num=1, top=top1))
             ]
             util.info(f'Design tops: {self.synth_designs_fpaths}')
 
         else:
-            # don't run synthesis, need the two top level .v files in
+            # don't run synthesis, need the two top level .v|.sv files in
             # self.synth_designs_fpaths, and need the two top module names in
             # self.synth_designs_tops
             self.synth_designs_fpaths = [
@@ -660,11 +674,19 @@ class CommandLecYosys(CommandLec, ToolYosys):
                 os.path.abspath(self.args['designs'][1])
             ]
 
+            for i in (0, 1):
+                if not os.path.isfile(self.args['designs'][i]):
+                    self.error(
+                        'Using synth=False (--no-synth) --designs=<value> must be a single',
+                        f'filename, however {self.args["designs"][i]} does not exist'
+                    )
+
             path, fname = os.path.split(self.synth_designs_fpaths[0])
             module_guess, _ = os.path.splitext(fname)
             top1 = util.get_inferred_top_module_name(
                 module_guess=module_guess, module_fpath=self.synth_designs_fpaths[0]
             )
+            util.debug(f'design1 {module_guess=} {fname=} {path=}')
             util.info(f'design1 top module name = {top1} (from {path} / {fname})')
 
             path, fname = os.path.split(self.synth_designs_fpaths[1])
@@ -672,6 +694,7 @@ class CommandLecYosys(CommandLec, ToolYosys):
             top2 = util.get_inferred_top_module_name(
                 module_guess=module_guess, module_fpath=self.synth_designs_fpaths[1]
             )
+            util.debug(f'design2 {module_guess=} {fname=} {path=}')
             util.info(f'design2 top module name = {top2} (from {path} / {fname})')
 
             self.synth_designs_tops = [top1, top2]
@@ -691,13 +714,20 @@ class CommandLecYosys(CommandLec, ToolYosys):
                     ]
                 else:
                     self.error(f' --pre-read-verilog file {x} does not exist')
+
+        nooverwrite = ''
+        if self.args['synth'] and not self.args['flatten-all']:
+            # If we don't flatten-all from synthesis, and we had to run synthesis,
+            # then read the 2nd file with -overwrite
+            nooverwrite = '-nooverwrite'
+
         lec_cmd_f_list += [
             '# Design1 (module):',
             f'read_verilog -sv -icells {self.synth_designs_fpaths[0]}',
             '# Design2 (module):',
-            f'read_verilog -sv -icells {self.synth_designs_fpaths[1]}',
+            f'read_verilog -sv -icells {self.synth_designs_fpaths[1]} {nooverwrite}',
             'clk2fflogic;',
-            f'miter -equiv -flatten {" ".join(self.synth_design_top_module_names)} miter',
+            f'miter -equiv -flatten {" ".join(self.synth_designs_tops)} miter',
             ('sat -seq 50 -verify -prove trigger 0 -show-all -show-inputs -show-outputs'
              ' -set-init-zero miter'),
         ]
