@@ -14,7 +14,7 @@ from opencos.util import debug
 
 vscode_path = shutil.which('code')
 
-INIT_HAS_RUN = False
+INIT_HAS_RUN = False # pylint: disable=invalid-name
 EXTENSIONS = {} # dict of {name: version} for VScode extensions of name
 
 
