@@ -10,7 +10,7 @@ import subprocess
 
 from opencos.util import debug
 
-INIT_HAS_RUN = False
+INIT_HAS_RUN = False  # pylint: disable=invalid-name
 TOOL_IS = {
     'riviera': False,
     'modelsim_ase': False,
